'' *****************************
'' GPS routines
''  (c) 2007 Perry James Mole
''  pjm@ridge-communications.ca
'' *****************************

' PVH Comment - Excellent small GPS reader routines.
' $GPRMC  Recommended minimum data                 ie: $GPRMC,081836,A,3751.6565,S,14507.3654,E,000.0,360.0,130998,011.3,E*62
' $GPGGA  GPS Fix Data                             ie: $GPGGA,170834,4124.8963,N,08151.6838,W,1,05,1.5,280.2,M,-34.0,M,,,*75
' $PGRMZ  eTrex proprietary barometric altitude ft ie: $PGRMZ,453,f,2*18
' v03 PVH Added GPS data "freshness" check to verify continuous gps data input.


CON
  CR = 13                                               ' ASCII <CR>
  LF = 10                                               ' ASCII <LF>
  
VAR  
   long gps_stack[30] 
   byte GPRMCb[80]
   long GPRMCa[80]  

   byte gps_buff[80],Rx',cksum
   long cog,cptr,arg,j
   long Null[1]

OBJ
  uart :  "Serial_Small"
  
PUB start(serRecv,serXmit) : okay

'' Starts uart object (at baud specified) in a cog
'' -- returns false if no cog available

  okay := uart.start(serRecv,serXmit,0,4800)            ' for USGlobeSat model 408 mode is 0 
                                                        ' for serial PC simulator  mode is 1
  return cog := cognew(readNEMA,@gps_stack) + 1 
    
PUB stop

'' Stop serial driver - frees a cog

  if cog
    cogstop(cog~ - 1)
  





PUB readNEMA 

  Null[0] := 0
  repeat
   bytefill(@gps_buff,0,79)     ' zeroes the memory space
   repeat while Rx <>= "$"      ' wait for the $ to insure we are starting with
     Rx := uart.rx              ' a complete NMEA sentence 
   cptr := 0

   repeat while Rx <>= CR and cptr=<79       '  continue to collect data until the end of the NMEA sentence 
     Rx := uart.rx              '  get character from Rx Buffer
     if Rx == ","
       gps_buff[cptr++] := 0    '  If "," replace the character with 0
     else
       gps_buff[cptr++] := Rx   '  else save the character   
   
   

   if gps_buff[2] == "R"             
     if gps_buff[3] == "M"            
       if gps_buff[4] == "C"           
           copy_buffer(@GPRMCb, @GPRMCa)
                   
   
                
pub copy_buffer ( buffer,args) |ptr
         bytemove(buffer,@gps_buff,cptr) '  copy received data to buffer
         ptr:=buffer
         arg := 0
         repeat j from 0 to 78           ' build array of pointers
          if byte[ptr] == 0               ' to each
             if byte[ptr+1] == 0           ' record
                long[args][arg] := Null     ' in 
             else                            ' the
                long[args][arg] := ptr+1      ' data buffer
             if arg=>29
                quit
             arg++
          ptr++
          
' now we just need to return the pointer to the desired record
          

pub valid
   return GPRMCa[1]
      
pub speed
   return GPRMCa[6]

pub heading
   return GPRMCa[7]
   
pub date
   return GPRMCa[8]
    






pub latitude
   return GPRMCa[2]
    
pub N_S
   return GPRMCa[3]
     
pub longitude
   return GPRMCa[4]
    
pub E_W
   return GPRMCa[5]



pub noGPSData
   GPRMCa.long[1] := string("X")

PUB crc8bitwise(pBuf, buflen, poly, initial): crc | i,k

'pBuf is the string to be CRCs
'buflen is thelength of th ebuffer to be CRCd
'poly is the ?
'initial is the inital value of CRC
 
  crc := initial
  repeat i from 0 to buflen - 1
    crc ^= (BYTE[pBuf][i] >< 8)
    repeat k from 0 to 7
      if crc & $80
        crc := (crc << 1) ^ poly
      else
        crc <<= 1
  crc &= $ff
  crc ><= 8